﻿using FitConnect.Models.Api.Metadata;
using FitConnect.Models.Api.Set;

namespace FitConnect.Builder;

public class PaymentInformationBuilder : IPaymentInformationBuilderWithGrossAmount,
    IPaymentInformationBuilderWithPaymentMethod, IPaymentInformationBuilderWithPaymentMethodDetail,
    IPaymentInformationBuilderWithStatus, IPaymentInformationBuilderWithTransactionId,
    IPaymentInformationBuilderWithTransactionReference,
    IPaymentInformationBuilderWithTransactionTimestamp,
    IPaymentInformationBuilderWithTransactionUrl {
    public double? GrossAmount { get; set; }

    public PaymentMethod? PaymentMethod { get; set; }
    public string? PaymentMethodDetail { get; set; }
    public PaymentStatus? Status { get; set; }
    public string? TransactionId { get; set; }
    public string? TransactionReference { get; set; }
    public DateTime? TransactionTimestamp { get; set; }

    public Uri? TransactionUrl { get; set; }

    public IPaymentInformationBuilderWithGrossAmount WithGrossAmount(double? value) {
        this.GrossAmount = value;
        return this;
    }

    public IPaymentInformationBuilderWithPaymentMethod WithPaymentMethod(PaymentMethod value) {
        this.PaymentMethod = value;
        return this;
    }

    public IPaymentInformationBuilderWithPaymentMethodDetail WithPaymentMethodDetail(string value) {
        this.PaymentMethodDetail = value;
        return this;
    }

    public IPaymentInformationBuilderWithStatus WithStatus(PaymentStatus value) {
        this.Status = value;
        return this;
    }

    public IPaymentInformationBuilderWithTransactionId WithTransactionId(string value) {
        this.TransactionId = value;
        return this;
    }

    public IPaymentInformationBuilderWithTransactionReference
        WithTransactionReference(string value) {
        this.TransactionReference = value;
        return this;
    }

    public IPaymentInformationBuilderWithTransactionTimestamp WithTransactionTimestamp(
        DateTime value) {
        this.TransactionTimestamp = value;
        return this;
    }

    public IPaymentInformationBuilderWithTransactionUrl WithTransactionUrl(Uri value) {
        return new PaymentInformationBuilder() {
            TransactionUrl = value
        };
    }

    public PaymentInformation Build() {
        if (PaymentMethod == null)
            throw new ArgumentNullException(nameof(PaymentMethod));
        if (Status == null)
            throw new ArgumentNullException(nameof(Status));
        if (TransactionId == null)
            throw new ArgumentNullException(nameof(TransactionId));
        if (TransactionReference == null)
            throw new ArgumentNullException(nameof(TransactionReference));


        return new PaymentInformation(
            PaymentMethod.Value,
            Status.Value,
            TransactionId,
            TransactionReference
        ) {
            GrossAmount = GrossAmount,
            TransactionTimestamp = TransactionTimestamp,
            TransactionUrl = TransactionUrl,
            PaymentMethodDetail = PaymentMethodDetail
        };
    }
}
