using Newtonsoft.Json;

namespace FitConnect.Models;

public class AuthenticationTags {
    [JsonProperty("metadata", NullValueHandling = NullValueHandling.Ignore)]
    public string? Metadata { get; set; }

    [JsonProperty("data", NullValueHandling = NullValueHandling.Ignore)]
    public string? Data { get; set; }

    [JsonProperty("attachments", NullValueHandling = NullValueHandling.Ignore)]
    public Dictionary<Guid, string>? Attachments { get; set; }
}

public static class AuthenticationTagExtensions {
    public static bool AreEqualTo(this AuthenticationTags expected, AuthenticationTags actual) {
        if (expected.Attachments == null && actual.Attachments == null) {
            return expected.Data == actual.Data && expected.Metadata == actual.Metadata;
        }

        if (actual.Attachments == null ^ expected.Attachments == null) {
            return false;
        }

        return expected.Metadata == actual.Metadata
               && expected.Data == actual.Data
               && expected.Attachments!.Any((item) =>
                   actual.Attachments!.ContainsKey(item.Key) &&
                   actual.Attachments[item.Key] == item.Value);
    }
}
