#nullable enable
using FitConnect.Models.Api.Set;
using Newtonsoft.Json;

namespace FitConnect.Models.Api.Metadata;

/// <summary>
/// Dieses Objekt enthält die Informationen vom Bezahldienst.
/// </summary>
public class PaymentInformation {
    private PaymentInformation() {
        TransactionId = string.Empty;
        TransactionReference = string.Empty;
    }

    public PaymentInformation(PaymentMethod paymentMethod, PaymentStatus status,
        string transactionId, string transactionReference) {
        PaymentMethod = paymentMethod;
        Status = status;
        TransactionId = transactionId;
        TransactionReference = transactionReference;
    }

    /// <summary>
    /// Bruttobetrag
    /// </summary>
    [JsonProperty("grossAmount", NullValueHandling = NullValueHandling.Ignore)]
    public double? GrossAmount { get; set; }

    /// <summary>
    /// Die vom Benutzer ausgewählte Zahlart. Das Feld ist nur bei einer erfolgreichen Zahlung
    /// vorhanden / befüllt.
    /// </summary>
    [JsonProperty("paymentMethod")]
    public PaymentMethod PaymentMethod { get; set; }

    /// <summary>
    /// Weitere Erläuterung zur gewählten Zahlart.
    /// </summary>
    [JsonProperty("paymentMethodDetail", NullValueHandling = NullValueHandling.Ignore)]

    public string? PaymentMethodDetail { get; set; }

    /// <summary>
    /// - INITIAL - der Einreichung hat einen Payment-Request ausgelöst und eine
    /// Payment-Transaction wurde angelegt. Der Nutzer hat aber im Bezahldienst noch keine
    /// Wirkung erzeugt.
    /// - BOOKED - der Nutzer hat die Bezahlung im Bezahldienst autorisiert.
    /// - FAILED - der Vorgang wurde vom Bezahldienst aufgrund der Nutzereingaben abgebrochen.
    /// - CANCELED - der Nutzer hat die Bezahlung im Bezahldienst abgebrochen.
    /// </summary>
    [JsonProperty("status")]
    public PaymentStatus Status { get; set; }

    /// <summary>
    /// Eine vom Bezahldienst vergebene Transaktions-Id.
    /// </summary>
    [JsonProperty("transactionId")]

    public string TransactionId { get; set; }

    /// <summary>
    /// Bezahlreferenz bzw. Verwendungszweck, wie z. B. ein Kassenzeichen.
    /// </summary>
    [JsonProperty("transactionReference")]
    public string TransactionReference { get; set; }

    /// <summary>
    /// Zeitstempel der erfolgreichen Durchführung der Bezahlung.
    /// </summary>
    [JsonProperty("transactionTimestamp", NullValueHandling = NullValueHandling.Ignore)]
    public DateTimeOffset? TransactionTimestamp { get; set; }

    /// <summary>
    /// Die Rest-URL der Payment Transaction für die Statusabfrage.
    /// </summary>
    [JsonProperty("transactionUrl", NullValueHandling = NullValueHandling.Ignore)]
    public Uri? TransactionUrl { get; set; }
}
